@@ -43,25 +43,6 @@
 #include "gen6_vme.h"
 #include "intel_media.h"
 
-#define BRC_CLIP(x, min, max)                                   \
-    {                                                           \
-        x = ((x > (max)) ? (max) : ((x < (min)) ? (min) : x));  \
-    }
-
-#define BRC_P_B_QP_DIFF 4
-#define BRC_I_P_QP_DIFF 2
-#define BRC_I_B_QP_DIFF (BRC_I_P_QP_DIFF + BRC_P_B_QP_DIFF)
-
-#define BRC_PWEIGHT 0.6  /* weight if P slice with comparison to I slice */
-#define BRC_BWEIGHT 0.25 /* weight if B slice with comparison to I slice */
-
-#define BRC_QP_MAX_CHANGE 5 /* maximum qp modification */
-#define BRC_CY 0.1 /* weight for */
-#define BRC_CX_UNDERFLOW 5.
-#define BRC_CX_OVERFLOW -4.
-
-#define BRC_PI_0_5 1.5707963267948966192313216916398
-
 #ifndef HAVE_LOG2F
 #define log2f(x) (logf(x)/(float)M_LN2)
 #endif
@@ -134,8 +115,8 @@ static void intel_mfc_brc_init(struct encode_state *encode_state,
 {
     struct gen6_mfc_context *mfc_context = encoder_context->mfc_context;
     VAEncSequenceParameterBufferH264 *pSequenceParameter = (VAEncSequenceParameterBufferH264 *)encode_state->seq_param_ext->buffer;
-    VAEncMiscParameterBuffer* pMiscParamHRD = (VAEncMiscParameterBuffer*)encode_state->misc_param[VAEncMiscParameterTypeHRD]->buffer;
-    VAEncMiscParameterHRD* pParameterHRD = (VAEncMiscParameterHRD*)pMiscParamHRD->data;
+    VAEncMiscParameterBuffer* pMiscParamHRD = NULL;
+    VAEncMiscParameterHRD* pParameterHRD = NULL;
     double bitrate = pSequenceParameter->bits_per_second;
     double framerate = (double)pSequenceParameter->time_scale /(2 * (double)pSequenceParameter->num_units_in_tick);
     int inum = 1, pnum = 0, bnum = 0; /* Gop structure: number of I, P, B frames in the Gop. */
@@ -145,6 +126,12 @@ static void intel_mfc_brc_init(struct encode_state *encode_state,
     double qp51_size = 0.001 * 8 * 3 * (pSequenceParameter->picture_width_in_mbs<<4) * (pSequenceParameter->picture_height_in_mbs<<4)/2;
     double bpf;
 
+    if (!encode_state->misc_param[VAEncMiscParameterTypeHRD] || !encode_state->misc_param[VAEncMiscParameterTypeHRD]->buffer)
+        return;
+
+    pMiscParamHRD = (VAEncMiscParameterBuffer*)encode_state->misc_param[VAEncMiscParameterTypeHRD]->buffer;
+    pParameterHRD = (VAEncMiscParameterHRD*)pMiscParamHRD->data;
+
     if (pSequenceParameter->ip_period) {
         pnum = (intra_period + ip_period - 1)/ip_period - 1;
         bnum = intra_period - inum - pnum;
@@ -631,8 +618,8 @@ VAStatus intel_mfc_avc_prepare(VADriverContextP ctx,
     int height_in_mbs = pSequenceParameter->picture_height_in_mbs;
 
     if (IS_GEN6(i965->intel.device_info)) {
-	/* On the SNB it should be fixed to 128 for the DMV buffer */
-	width_in_mbs = 128;
+        /* On the SNB it should be fixed to 128 for the DMV buffer */
+        width_in_mbs = 128;
     }
 
     for (j = 0; j < encode_state->num_slice_params_ext && enable_avc_ildb == 0; j++) {
@@ -735,7 +722,7 @@ VAStatus intel_mfc_avc_prepare(VADriverContextP ctx,
             break;
         }
     }
-	
+
     mfc_context->uncompressed_picture_source.bo = encode_state->input_yuv_object->bo;
     dri_bo_reference(mfc_context->uncompressed_picture_source.bo);
 
@@ -807,11 +794,12 @@ int intel_format_lutvalue(int value, int max)
     if (temp1 > temp2)
         ret = max;
     return ret;
-	
+
 }
 
 
 #define		QP_MAX			52
+#define		VP8_QP_MAX	        128
 
 
 static float intel_lambda_qp(int qp)
@@ -842,40 +830,40 @@ void intel_vme_update_mbmv_cost(VADriverContextP ctx,
 
     
     if (encoder_context->rate_control_mode == VA_RC_CQP)
-	qp = pic_param->pic_init_qp + slice_param->slice_qp_delta;
+        qp = pic_param->pic_init_qp + slice_param->slice_qp_delta;
     else
-	qp = mfc_context->bit_rate_control_context[slice_type].QpPrimeY;
-  
+        qp = mfc_context->bit_rate_control_context[slice_type].QpPrimeY;
+
     if (vme_state_message == NULL)
-	return;
- 
+        return;
+
     assert(qp <= QP_MAX); 
     lambda = intel_lambda_qp(qp);
     if (slice_type == SLICE_TYPE_I) {
-	vme_state_message[MODE_INTRA_16X16] = 0;
-	m_cost = lambda * 4;
-	vme_state_message[MODE_INTRA_8X8] = intel_format_lutvalue(m_cost, 0x8f);
-	m_cost = lambda * 16; 
-	vme_state_message[MODE_INTRA_4X4] = intel_format_lutvalue(m_cost, 0x8f);
-	m_cost = lambda * 3;
-	vme_state_message[MODE_INTRA_NONPRED] = intel_format_lutvalue(m_cost, 0x6f);
+        vme_state_message[MODE_INTRA_16X16] = 0;
+        m_cost = lambda * 4;
+        vme_state_message[MODE_INTRA_8X8] = intel_format_lutvalue(m_cost, 0x8f);
+        m_cost = lambda * 16; 
+        vme_state_message[MODE_INTRA_4X4] = intel_format_lutvalue(m_cost, 0x8f);
+        m_cost = lambda * 3;
+        vme_state_message[MODE_INTRA_NONPRED] = intel_format_lutvalue(m_cost, 0x6f);
     } else {
-    	m_cost = 0;
-	vme_state_message[MODE_INTER_MV0] = intel_format_lutvalue(m_cost, 0x6f);
-	for (j = 1; j < 3; j++) {
+        m_cost = 0;
+        vme_state_message[MODE_INTER_MV0] = intel_format_lutvalue(m_cost, 0x6f);
+        for (j = 1; j < 3; j++) {
             m_costf = (log2f((float)(j + 1)) + 1.718f) * lambda;
             m_cost = (int)m_costf;
             vme_state_message[MODE_INTER_MV0 + j] = intel_format_lutvalue(m_cost, 0x6f);
-   	}
-    	mv_count = 3;
-    	for (j = 4; j <= 64; j *= 2) {
+        }
+        mv_count = 3;
+        for (j = 4; j <= 64; j *= 2) {
             m_costf = (log2f((float)(j + 1)) + 1.718f) * lambda;
             m_cost = (int)m_costf;
             vme_state_message[MODE_INTER_MV0 + mv_count] = intel_format_lutvalue(m_cost, 0x6f);
             mv_count++;
-	}
+        }
 
-	if (qp <= 25) {
+        if (qp <= 25) {
             vme_state_message[MODE_INTRA_16X16] = 0x4a;
             vme_state_message[MODE_INTRA_8X8] = 0x4a;
             vme_state_message[MODE_INTRA_4X4] = 0x4a;
@@ -887,17 +875,17 @@ void intel_vme_update_mbmv_cost(VADriverContextP ctx,
             vme_state_message[MODE_INTER_4X4] = 0x4a;
             vme_state_message[MODE_INTER_BWD] = 0x2a;
             return;
-	}
-	m_costf = lambda * 10;
-	vme_state_message[MODE_INTRA_16X16] = intel_format_lutvalue(m_cost, 0x8f);
-	m_cost = lambda * 14;
-	vme_state_message[MODE_INTRA_8X8] = intel_format_lutvalue(m_cost, 0x8f);
-	m_cost = lambda * 24; 
-	vme_state_message[MODE_INTRA_4X4] = intel_format_lutvalue(m_cost, 0x8f);
-	m_costf = lambda * 3.5;
-	m_cost = m_costf;
-	vme_state_message[MODE_INTRA_NONPRED] = intel_format_lutvalue(m_cost, 0x6f);
-    	if (slice_type == SLICE_TYPE_P) {
+        }
+        m_costf = lambda * 10;
+        vme_state_message[MODE_INTRA_16X16] = intel_format_lutvalue(m_cost, 0x8f);
+        m_cost = lambda * 14;
+        vme_state_message[MODE_INTRA_8X8] = intel_format_lutvalue(m_cost, 0x8f);
+        m_cost = lambda * 24; 
+        vme_state_message[MODE_INTRA_4X4] = intel_format_lutvalue(m_cost, 0x8f);
+        m_costf = lambda * 3.5;
+        m_cost = m_costf;
+        vme_state_message[MODE_INTRA_NONPRED] = intel_format_lutvalue(m_cost, 0x6f);
+        if (slice_type == SLICE_TYPE_P) {
             m_costf = lambda * 2.5;
             m_cost = m_costf;
             vme_state_message[MODE_INTER_16X16] = intel_format_lutvalue(m_cost, 0x8f);
@@ -915,7 +903,7 @@ void intel_vme_update_mbmv_cost(VADriverContextP ctx,
             vme_state_message[MODE_INTER_4X4] = intel_format_lutvalue(m_cost, 0x6f);
             /* BWD is not used in P-frame */
             vme_state_message[MODE_INTER_BWD] = 0;
-	} else {
+        } else {
             m_costf = lambda * 2.5;
             m_cost = m_costf;
             vme_state_message[MODE_INTER_16X16] = intel_format_lutvalue(m_cost, 0x8f);
@@ -934,10 +922,85 @@ void intel_vme_update_mbmv_cost(VADriverContextP ctx,
             m_costf = lambda * 1.5;
             m_cost = m_costf;
             vme_state_message[MODE_INTER_BWD] = intel_format_lutvalue(m_cost, 0x6f);
-	}
+        }
     }
 }
 
+void intel_vme_vp8_update_mbmv_cost(VADriverContextP ctx,
+                                struct encode_state *encode_state,
+                                struct intel_encoder_context *encoder_context)
+{
+    struct gen6_mfc_context *mfc_context = encoder_context->mfc_context;
+    struct gen6_vme_context *vme_context = encoder_context->vme_context;
+    VAEncPictureParameterBufferVP8 *pic_param = (VAEncPictureParameterBufferVP8 *)encode_state->pic_param_ext->buffer;
+    VAQMatrixBufferVP8 *q_matrix = (VAQMatrixBufferVP8 *)encode_state->q_matrix->buffer;
+    int qp, m_cost, j, mv_count;
+    uint8_t *vme_state_message = (uint8_t *)(vme_context->vme_state_message);
+    float   lambda, m_costf;
+
+    int is_key_frame = !pic_param->pic_flags.bits.frame_type;
+    int slice_type = (is_key_frame ? SLICE_TYPE_I : SLICE_TYPE_P);
+  
+    if (vme_state_message == NULL)
+	return;
+ 
+    if (encoder_context->rate_control_mode == VA_RC_CQP)
+        qp = q_matrix->quantization_index[0];
+    else
+        qp = mfc_context->bit_rate_control_context[slice_type].QpPrimeY;
+
+    lambda = intel_lambda_qp(qp * QP_MAX / VP8_QP_MAX);
+    if (is_key_frame) {
+	vme_state_message[MODE_INTRA_16X16] = 0;
+	m_cost = lambda * 16; 
+	vme_state_message[MODE_INTRA_4X4] = intel_format_lutvalue(m_cost, 0x8f);
+    } else {
+    	m_cost = 0;
+	vme_state_message[MODE_INTER_MV0] = intel_format_lutvalue(m_cost, 0x6f);
+	for (j = 1; j < 3; j++) {
+            m_costf = (log2f((float)(j + 1)) + 1.718f) * lambda;
+            m_cost = (int)m_costf;
+            vme_state_message[MODE_INTER_MV0 + j] = intel_format_lutvalue(m_cost, 0x6f);
+   	}
+    	mv_count = 3;
+    	for (j = 4; j <= 64; j *= 2) {
+            m_costf = (log2f((float)(j + 1)) + 1.718f) * lambda;
+            m_cost = (int)m_costf;
+            vme_state_message[MODE_INTER_MV0 + mv_count] = intel_format_lutvalue(m_cost, 0x6f);
+            mv_count++;
+	}
+
+	if (qp < 92 ) {
+            vme_state_message[MODE_INTRA_16X16] = 0x4a;
+            vme_state_message[MODE_INTRA_4X4] = 0x4a;
+            vme_state_message[MODE_INTRA_NONPRED] = 0x4a;
+            vme_state_message[MODE_INTER_16X16] = 0x4a;
+            vme_state_message[MODE_INTER_16X8] = 0x4a;
+            vme_state_message[MODE_INTER_8X8] = 0x4a;
+            vme_state_message[MODE_INTER_4X4] = 0x4a;
+            return;
+	}
+	m_costf = lambda * 10;
+	vme_state_message[MODE_INTRA_16X16] = intel_format_lutvalue(m_cost, 0x8f);
+	m_cost = lambda * 24; 
+	vme_state_message[MODE_INTRA_4X4] = intel_format_lutvalue(m_cost, 0x8f);
+            
+        m_costf = lambda * 2.5;
+        m_cost = m_costf;
+        vme_state_message[MODE_INTER_16X16] = intel_format_lutvalue(m_cost, 0x8f);
+        m_costf = lambda * 4;
+        m_cost = m_costf;
+        vme_state_message[MODE_INTER_16X8] = intel_format_lutvalue(m_cost, 0x8f);
+        m_costf = lambda * 1.5;
+        m_cost = m_costf;
+        vme_state_message[MODE_INTER_8X8] = intel_format_lutvalue(m_cost, 0x6f);
+        m_costf = lambda * 5;
+        m_cost = m_costf;
+        vme_state_message[MODE_INTER_4X4] = intel_format_lutvalue(m_cost, 0x6f);
+        /* BWD is not used in P-frame */
+        vme_state_message[MODE_INTER_BWD] = 0;
+    }
+}
 
 #define		MB_SCOREBOARD_A		(1 << 0)
 #define		MB_SCOREBOARD_B		(1 << 1)
@@ -961,7 +1024,7 @@ gen7_vme_scoreboard_init(VADriverContextP ctx, struct gen6_vme_context *vme_cont
     vme_context->gpe_context.vfe_desc6.scoreboard1.delta_y1 = -1;
     vme_context->gpe_context.vfe_desc6.scoreboard1.delta_x2 = 1;
     vme_context->gpe_context.vfe_desc6.scoreboard1.delta_y2 = -1;
-	
+
     vme_context->gpe_context.vfe_desc7.dword = 0;
     return;
 }
@@ -974,7 +1037,7 @@ static inline int loop_in_bounds(int x_index, int y_index, int first_mb, int num
         return -1;
     if (y_index < 0 || y_index >= mb_height)
         return -1;
-	
+
     mb_index = y_index * mb_width + x_index;
     if (mb_index < first_mb || mb_index > (first_mb + num_mb))
         return -1;
@@ -1000,103 +1063,103 @@ gen7_vme_walker_fill_vme_batchbuffer(VADriverContextP ctx,
     command_ptr = vme_context->vme_batchbuffer.bo->virtual;
 
     for (s = 0; s < encode_state->num_slice_params_ext; s++) {
-	VAEncSliceParameterBufferH264 *pSliceParameter = (VAEncSliceParameterBufferH264 *)encode_state->slice_params_ext[s]->buffer;
-	int first_mb = pSliceParameter->macroblock_address;
-	int num_mb = pSliceParameter->num_macroblocks;
-	unsigned int mb_intra_ub, score_dep;
-	int x_outer, y_outer, x_inner, y_inner;
-	int xtemp_outer = 0;
+        VAEncSliceParameterBufferH264 *pSliceParameter = (VAEncSliceParameterBufferH264 *)encode_state->slice_params_ext[s]->buffer;
+        int first_mb = pSliceParameter->macroblock_address;
+        int num_mb = pSliceParameter->num_macroblocks;
+        unsigned int mb_intra_ub, score_dep;
+        int x_outer, y_outer, x_inner, y_inner;
+        int xtemp_outer = 0;
 
-	x_outer = first_mb % mb_width;
-	y_outer = first_mb / mb_width;
-	mb_row = y_outer;
-				 
-	for (; x_outer < (mb_width -2 ) && !loop_in_bounds(x_outer, y_outer, first_mb, num_mb, mb_width, mb_height); ) {
-	    x_inner = x_outer;
-	    y_inner = y_outer;
-	    for (; !loop_in_bounds(x_inner, y_inner, first_mb, num_mb, mb_width, mb_height);) {
-		mb_intra_ub = 0;
-		score_dep = 0;
-		if (x_inner != 0) {
-		    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_AE;
-		    score_dep |= MB_SCOREBOARD_A; 
-		}
-		if (y_inner != mb_row) {
-		    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_B;
-		    score_dep |= MB_SCOREBOARD_B;
-		    if (x_inner != 0)
-			mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_D;
-		    if (x_inner != (mb_width -1)) {
-			mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_C;
-			score_dep |= MB_SCOREBOARD_C;
+        x_outer = first_mb % mb_width;
+        y_outer = first_mb / mb_width;
+        mb_row = y_outer;
+
+        for (; x_outer < (mb_width -2 ) && !loop_in_bounds(x_outer, y_outer, first_mb, num_mb, mb_width, mb_height); ) {
+            x_inner = x_outer;
+            y_inner = y_outer;
+            for (; !loop_in_bounds(x_inner, y_inner, first_mb, num_mb, mb_width, mb_height);) {
+                mb_intra_ub = 0;
+                score_dep = 0;
+                if (x_inner != 0) {
+                    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_AE;
+                    score_dep |= MB_SCOREBOARD_A; 
+                }
+                if (y_inner != mb_row) {
+                    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_B;
+                    score_dep |= MB_SCOREBOARD_B;
+                    if (x_inner != 0)
+                        mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_D;
+                    if (x_inner != (mb_width -1)) {
+                        mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_C;
+                        score_dep |= MB_SCOREBOARD_C;
                     }
-		}
-							
-            	*command_ptr++ = (CMD_MEDIA_OBJECT | (8 - 2));
-		*command_ptr++ = kernel;
-		*command_ptr++ = USE_SCOREBOARD;
-		/* Indirect data */
-		*command_ptr++ = 0;
-		/* the (X, Y) term of scoreboard */
-		*command_ptr++ = ((y_inner << 16) | x_inner);
-		*command_ptr++ = score_dep;
-		/*inline data */
-		*command_ptr++ = (mb_width << 16 | y_inner << 8 | x_inner);
-		*command_ptr++ = ((1 << 18) | (1 << 16) | transform_8x8_mode_flag | (mb_intra_ub << 8));
-		x_inner -= 2;
-		y_inner += 1;
-	    }
-	    x_outer += 1;
-	}
+                }
 
-	xtemp_outer = mb_width - 2;
-	if (xtemp_outer < 0)
+                *command_ptr++ = (CMD_MEDIA_OBJECT | (8 - 2));
+                *command_ptr++ = kernel;
+                *command_ptr++ = USE_SCOREBOARD;
+                /* Indirect data */
+                *command_ptr++ = 0;
+                /* the (X, Y) term of scoreboard */
+                *command_ptr++ = ((y_inner << 16) | x_inner);
+                *command_ptr++ = score_dep;
+                /*inline data */
+                *command_ptr++ = (mb_width << 16 | y_inner << 8 | x_inner);
+                *command_ptr++ = ((1 << 18) | (1 << 16) | transform_8x8_mode_flag | (mb_intra_ub << 8));
+                x_inner -= 2;
+                y_inner += 1;
+            }
+            x_outer += 1;
+        }
+
+        xtemp_outer = mb_width - 2;
+        if (xtemp_outer < 0)
             xtemp_outer = 0;
-	x_outer = xtemp_outer;
-	y_outer = first_mb / mb_width;
-	for (;!loop_in_bounds(x_outer, y_outer, first_mb, num_mb, mb_width, mb_height); ) { 
-	    y_inner = y_outer;
-	    x_inner = x_outer;
-	    for (; !loop_in_bounds(x_inner, y_inner, first_mb, num_mb, mb_width, mb_height);) {
-	    	mb_intra_ub = 0;
-		score_dep = 0;
-		if (x_inner != 0) {
-		    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_AE;
-		    score_dep |= MB_SCOREBOARD_A; 
-		}
-		if (y_inner != mb_row) {
-		    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_B;
-		    score_dep |= MB_SCOREBOARD_B;
-		    if (x_inner != 0)
-			mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_D;
+        x_outer = xtemp_outer;
+        y_outer = first_mb / mb_width;
+        for (;!loop_in_bounds(x_outer, y_outer, first_mb, num_mb, mb_width, mb_height); ) { 
+            y_inner = y_outer;
+            x_inner = x_outer;
+            for (; !loop_in_bounds(x_inner, y_inner, first_mb, num_mb, mb_width, mb_height);) {
+                mb_intra_ub = 0;
+                score_dep = 0;
+                if (x_inner != 0) {
+                    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_AE;
+                    score_dep |= MB_SCOREBOARD_A; 
+                }
+                if (y_inner != mb_row) {
+                    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_B;
+                    score_dep |= MB_SCOREBOARD_B;
+                    if (x_inner != 0)
+                        mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_D;
 
-		    if (x_inner != (mb_width -1)) {
-			mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_C;
-			score_dep |= MB_SCOREBOARD_C;
+                    if (x_inner != (mb_width -1)) {
+                        mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_C;
+                        score_dep |= MB_SCOREBOARD_C;
                     }
-		}
+                }
 
-            	*command_ptr++ = (CMD_MEDIA_OBJECT | (8 - 2));
-		*command_ptr++ = kernel;
-		*command_ptr++ = USE_SCOREBOARD;
-		/* Indirect data */
-		*command_ptr++ = 0;
-		/* the (X, Y) term of scoreboard */
-		*command_ptr++ = ((y_inner << 16) | x_inner);
-		*command_ptr++ = score_dep;
-		/*inline data */
-		*command_ptr++ = (mb_width << 16 | y_inner << 8 | x_inner);
-		*command_ptr++ = ((1 << 18) | (1 << 16) | transform_8x8_mode_flag | (mb_intra_ub << 8));
+                *command_ptr++ = (CMD_MEDIA_OBJECT | (8 - 2));
+                *command_ptr++ = kernel;
+                *command_ptr++ = USE_SCOREBOARD;
+                /* Indirect data */
+                *command_ptr++ = 0;
+                /* the (X, Y) term of scoreboard */
+                *command_ptr++ = ((y_inner << 16) | x_inner);
+                *command_ptr++ = score_dep;
+                /*inline data */
+                *command_ptr++ = (mb_width << 16 | y_inner << 8 | x_inner);
+                *command_ptr++ = ((1 << 18) | (1 << 16) | transform_8x8_mode_flag | (mb_intra_ub << 8));
 
-		x_inner -= 2;
-		y_inner += 1;
-	    }
-	    x_outer++;
-	    if (x_outer >= mb_width) {
-		y_outer += 1;
-		x_outer = xtemp_outer;
-	    }		
-	}
+                x_inner -= 2;
+                y_inner += 1;
+            }
+            x_outer++;
+            if (x_outer >= mb_width) {
+                y_outer += 1;
+                x_outer = xtemp_outer;
+            }		
+        }
     }
 
     *command_ptr++ = 0;
@@ -1270,7 +1333,7 @@ void intel_vme_mpeg2_state_setup(VADriverContextP ctx,
          */
         vme_state_message[MODE_INTRA_16X16] = intel_format_lutvalue(m_cost, 0x8f);
         vme_state_message[MODE_INTER_16X16] = intel_format_lutvalue(m_cost, 0x8f);
-			
+
         vme_state_message[MODE_INTER_16X8] = 0;
         vme_state_message[MODE_INTER_8X8] = 0;
         vme_state_message[MODE_INTER_8X4] = 0;
@@ -1300,105 +1363,105 @@ gen7_vme_mpeg2_walker_fill_vme_batchbuffer(VADriverContextP ctx,
     command_ptr = vme_context->vme_batchbuffer.bo->virtual;
 
     {
-	unsigned int mb_intra_ub, score_dep;
-	int x_outer, y_outer, x_inner, y_inner;
-	int xtemp_outer = 0;
-	int first_mb = 0;
-	int num_mb = mb_width * mb_height;
+        unsigned int mb_intra_ub, score_dep;
+        int x_outer, y_outer, x_inner, y_inner;
+        int xtemp_outer = 0;
+        int first_mb = 0;
+        int num_mb = mb_width * mb_height;
 
-	x_outer = 0;
-	y_outer = 0;
-	
-				 
-	for (; x_outer < (mb_width -2 ) && !loop_in_bounds(x_outer, y_outer, first_mb, num_mb, mb_width, mb_height); ) {
-	    x_inner = x_outer;
-	    y_inner = y_outer;
-	    for (; !loop_in_bounds(x_inner, y_inner, first_mb, num_mb, mb_width, mb_height);) {
-		mb_intra_ub = 0;
-		score_dep = 0;
-		if (x_inner != 0) {
-		    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_AE;
-		    score_dep |= MB_SCOREBOARD_A; 
-		}
-		if (y_inner != 0) {
-		    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_B;
-		    score_dep |= MB_SCOREBOARD_B;
+        x_outer = 0;
+        y_outer = 0;
 
-		    if (x_inner != 0)
-			mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_D;
 
-		    if (x_inner != (mb_width -1)) {
-			mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_C;
-			score_dep |= MB_SCOREBOARD_C;
-		    }
-		}
-							
-            	*command_ptr++ = (CMD_MEDIA_OBJECT | (8 - 2));
-		*command_ptr++ = kernel;
-		*command_ptr++ = MPEG2_SCOREBOARD;
-		/* Indirect data */
-		*command_ptr++ = 0;
-		/* the (X, Y) term of scoreboard */
-		*command_ptr++ = ((y_inner << 16) | x_inner);
-		*command_ptr++ = score_dep;
-		/*inline data */
-		*command_ptr++ = (mb_width << 16 | y_inner << 8 | x_inner);
-		*command_ptr++ = ((1 << 18) | (1 << 16) | (mb_intra_ub << 8));
-		x_inner -= 2;
-		y_inner += 1;
-	    }
-	    x_outer += 1;
-	}
+        for (; x_outer < (mb_width -2 ) && !loop_in_bounds(x_outer, y_outer, first_mb, num_mb, mb_width, mb_height); ) {
+            x_inner = x_outer;
+            y_inner = y_outer;
+            for (; !loop_in_bounds(x_inner, y_inner, first_mb, num_mb, mb_width, mb_height);) {
+                mb_intra_ub = 0;
+                score_dep = 0;
+                if (x_inner != 0) {
+                    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_AE;
+                    score_dep |= MB_SCOREBOARD_A; 
+                }
+                if (y_inner != 0) {
+                    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_B;
+                    score_dep |= MB_SCOREBOARD_B;
 
-	xtemp_outer = mb_width - 2;
-	if (xtemp_outer < 0)
+                    if (x_inner != 0)
+                        mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_D;
+
+                    if (x_inner != (mb_width -1)) {
+                        mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_C;
+                        score_dep |= MB_SCOREBOARD_C;
+                    }
+                }
+
+                *command_ptr++ = (CMD_MEDIA_OBJECT | (8 - 2));
+                *command_ptr++ = kernel;
+                *command_ptr++ = MPEG2_SCOREBOARD;
+                /* Indirect data */
+                *command_ptr++ = 0;
+                /* the (X, Y) term of scoreboard */
+                *command_ptr++ = ((y_inner << 16) | x_inner);
+                *command_ptr++ = score_dep;
+                /*inline data */
+                *command_ptr++ = (mb_width << 16 | y_inner << 8 | x_inner);
+                *command_ptr++ = ((1 << 18) | (1 << 16) | (mb_intra_ub << 8));
+                x_inner -= 2;
+                y_inner += 1;
+            }
+            x_outer += 1;
+        }
+
+        xtemp_outer = mb_width - 2;
+        if (xtemp_outer < 0)
             xtemp_outer = 0;
-	x_outer = xtemp_outer;
-	y_outer = 0;
-	for (;!loop_in_bounds(x_outer, y_outer, first_mb, num_mb, mb_width, mb_height); ) { 
-	    y_inner = y_outer;
-	    x_inner = x_outer;
-	    for (; !loop_in_bounds(x_inner, y_inner, first_mb, num_mb, mb_width, mb_height);) {
-	    	mb_intra_ub = 0;
-		score_dep = 0;
-		if (x_inner != 0) {
-		    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_AE;
-		    score_dep |= MB_SCOREBOARD_A; 
-		}
-		if (y_inner != 0) {
-		    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_B;
-		    score_dep |= MB_SCOREBOARD_B;
+        x_outer = xtemp_outer;
+        y_outer = 0;
+        for (;!loop_in_bounds(x_outer, y_outer, first_mb, num_mb, mb_width, mb_height); ) { 
+            y_inner = y_outer;
+            x_inner = x_outer;
+            for (; !loop_in_bounds(x_inner, y_inner, first_mb, num_mb, mb_width, mb_height);) {
+                mb_intra_ub = 0;
+                score_dep = 0;
+                if (x_inner != 0) {
+                    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_AE;
+                    score_dep |= MB_SCOREBOARD_A; 
+                }
+                if (y_inner != 0) {
+                    mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_B;
+                    score_dep |= MB_SCOREBOARD_B;
 
-		    if (x_inner != 0)
-			mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_D;
+                    if (x_inner != 0)
+                        mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_D;
 
-		    if (x_inner != (mb_width -1)) {
-			mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_C;
-			score_dep |= MB_SCOREBOARD_C;
-		    }
-		}
+                    if (x_inner != (mb_width -1)) {
+                        mb_intra_ub |= INTRA_PRED_AVAIL_FLAG_C;
+                        score_dep |= MB_SCOREBOARD_C;
+                    }
+                }
 
-            	*command_ptr++ = (CMD_MEDIA_OBJECT | (8 - 2));
-		*command_ptr++ = kernel;
-		*command_ptr++ = MPEG2_SCOREBOARD;
-		/* Indirect data */
-		*command_ptr++ = 0;
-		/* the (X, Y) term of scoreboard */
-		*command_ptr++ = ((y_inner << 16) | x_inner);
-		*command_ptr++ = score_dep;
-		/*inline data */
-		*command_ptr++ = (mb_width << 16 | y_inner << 8 | x_inner);
-		*command_ptr++ = ((1 << 18) | (1 << 16) | (mb_intra_ub << 8));
+                *command_ptr++ = (CMD_MEDIA_OBJECT | (8 - 2));
+                *command_ptr++ = kernel;
+                *command_ptr++ = MPEG2_SCOREBOARD;
+                /* Indirect data */
+                *command_ptr++ = 0;
+                /* the (X, Y) term of scoreboard */
+                *command_ptr++ = ((y_inner << 16) | x_inner);
+                *command_ptr++ = score_dep;
+                /*inline data */
+                *command_ptr++ = (mb_width << 16 | y_inner << 8 | x_inner);
+                *command_ptr++ = ((1 << 18) | (1 << 16) | (mb_intra_ub << 8));
 
-		x_inner -= 2;
-		y_inner += 1;
-	    }
-	    x_outer++;
-	    if (x_outer >= mb_width) {
-		y_outer += 1;
-		x_outer = xtemp_outer;
-	    }		
-	}
+                x_inner -= 2;
+                y_inner += 1;
+            }
+            x_outer++;
+            if (x_outer >= mb_width) {
+                y_outer += 1;
+                x_outer = xtemp_outer;
+            }		
+        }
     }
 
     *command_ptr++ = 0;
@@ -1618,3 +1681,226 @@ void intel_avc_slice_insert_packed_data(VADriverContextP ctx,
 
     return;
 }
+
+/* HEVC */
+static int
+hevc_temporal_find_surface(VAPictureHEVC *curr_pic,
+                           VAPictureHEVC *ref_list,
+                           int num_pictures,
+                           int dir)
+{
+    int i, found = -1, min = 0x7FFFFFFF;
+
+    for (i = 0; i < num_pictures; i++) {
+        int tmp;
+
+        if ((ref_list[i].flags & VA_PICTURE_HEVC_INVALID) ||
+            (ref_list[i].picture_id == VA_INVALID_SURFACE))
+            break;
+
+        tmp = curr_pic->pic_order_cnt - ref_list[i].pic_order_cnt;
+
+        if (dir)
+            tmp = -tmp;
+
+        if (tmp > 0 && tmp < min) {
+            min = tmp;
+            found = i;
+        }
+    }
+
+    return found;
+}
+void
+intel_hevc_vme_reference_state(VADriverContextP ctx,
+                               struct encode_state *encode_state,
+                               struct intel_encoder_context *encoder_context,
+                               int list_index,
+                               int surface_index,
+                               void (* vme_source_surface_state)(
+                                   VADriverContextP ctx,
+                                   int index,
+                                   struct object_surface *obj_surface,
+                                   struct intel_encoder_context *encoder_context))
+{
+    struct gen6_vme_context *vme_context = encoder_context->vme_context;
+    struct object_surface *obj_surface = NULL;
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    VASurfaceID ref_surface_id;
+    VAEncPictureParameterBufferHEVC *pic_param = (VAEncPictureParameterBufferHEVC *)encode_state->pic_param_ext->buffer;
+    VAEncSliceParameterBufferHEVC *slice_param = (VAEncSliceParameterBufferHEVC *)encode_state->slice_params_ext[0]->buffer;
+    int max_num_references;
+    VAPictureHEVC *curr_pic;
+    VAPictureHEVC *ref_list;
+    int ref_idx;
+
+    if (list_index == 0) {
+        max_num_references = pic_param->num_ref_idx_l0_default_active_minus1 + 1;
+        ref_list = slice_param->ref_pic_list0;
+    } else {
+        max_num_references = pic_param->num_ref_idx_l1_default_active_minus1 + 1;
+        ref_list = slice_param->ref_pic_list1;
+    }
+
+    if (max_num_references == 1) {
+        if (list_index == 0) {
+            ref_surface_id = slice_param->ref_pic_list0[0].picture_id;
+            vme_context->used_references[0] = &slice_param->ref_pic_list0[0];
+        } else {
+            ref_surface_id = slice_param->ref_pic_list1[0].picture_id;
+            vme_context->used_references[1] = &slice_param->ref_pic_list1[0];
+        }
+
+        if (ref_surface_id != VA_INVALID_SURFACE)
+            obj_surface = SURFACE(ref_surface_id);
+
+        if (!obj_surface ||
+            !obj_surface->bo) {
+            obj_surface = encode_state->reference_objects[list_index];
+            vme_context->used_references[list_index] = &pic_param->reference_frames[list_index];
+        }
+
+        ref_idx = 0;
+    } else {
+        curr_pic = &pic_param->decoded_curr_pic;
+
+        /* select the reference frame in temporal space */
+        ref_idx = hevc_temporal_find_surface(curr_pic, ref_list, max_num_references, list_index == 1);
+        ref_surface_id = ref_list[ref_idx].picture_id;
+
+        if (ref_surface_id != VA_INVALID_SURFACE) /* otherwise warning later */
+            obj_surface = SURFACE(ref_surface_id);
+
+        vme_context->used_reference_objects[list_index] = obj_surface;
+        vme_context->used_references[list_index] = &ref_list[ref_idx];
+    }
+
+    if (obj_surface &&
+        obj_surface->bo) {
+        assert(ref_idx >= 0);
+        vme_context->used_reference_objects[list_index] = obj_surface;
+        vme_source_surface_state(ctx, surface_index, obj_surface, encoder_context);
+        vme_context->ref_index_in_mb[list_index] = (ref_idx << 24 |
+                ref_idx << 16 |
+                ref_idx <<  8 |
+                ref_idx);
+    } else {
+        vme_context->used_reference_objects[list_index] = NULL;
+        vme_context->used_references[list_index] = NULL;
+        vme_context->ref_index_in_mb[list_index] = 0;
+    }
+}
+
+void intel_vme_hevc_update_mbmv_cost(VADriverContextP ctx,
+                                     struct encode_state *encode_state,
+                                     struct intel_encoder_context *encoder_context)
+{
+    //struct gen6_mfc_context *mfc_context = encoder_context->mfc_context;
+    struct gen6_vme_context *vme_context = encoder_context->vme_context;
+    VAEncPictureParameterBufferHEVC *pic_param = (VAEncPictureParameterBufferHEVC *)encode_state->pic_param_ext->buffer;
+    VAEncSliceParameterBufferHEVC *slice_param = (VAEncSliceParameterBufferHEVC *)encode_state->slice_params_ext[0]->buffer;
+    int qp, m_cost, j, mv_count;
+    uint8_t *vme_state_message = (uint8_t *)(vme_context->vme_state_message);
+    float   lambda, m_costf;
+
+    /* here no SI SP slice for HEVC, do not need slice fixup */
+    int slice_type = slice_param->slice_type;
+
+
+    /* to do for CBR*/
+    //if (encoder_context->rate_control_mode == VA_RC_CQP)
+    qp = pic_param->pic_init_qp + slice_param->slice_qp_delta;
+    //else
+    //qp = mfc_context->bit_rate_control_context[slice_type].QpPrimeY;
+
+    if (vme_state_message == NULL)
+        return;
+
+    assert(qp <= QP_MAX);
+    lambda = intel_lambda_qp(qp);
+    if (slice_type == HEVC_SLICE_I) {
+        vme_state_message[MODE_INTRA_16X16] = 0;
+        m_cost = lambda * 4;
+        vme_state_message[MODE_INTRA_8X8] = intel_format_lutvalue(m_cost, 0x8f);
+        m_cost = lambda * 16;
+        vme_state_message[MODE_INTRA_4X4] = intel_format_lutvalue(m_cost, 0x8f);
+        m_cost = lambda * 3;
+        vme_state_message[MODE_INTRA_NONPRED] = intel_format_lutvalue(m_cost, 0x6f);
+    } else {
+        m_cost = 0;
+        vme_state_message[MODE_INTER_MV0] = intel_format_lutvalue(m_cost, 0x6f);
+        for (j = 1; j < 3; j++) {
+            m_costf = (log2f((float)(j + 1)) + 1.718f) * lambda;
+            m_cost = (int)m_costf;
+            vme_state_message[MODE_INTER_MV0 + j] = intel_format_lutvalue(m_cost, 0x6f);
+        }
+        mv_count = 3;
+        for (j = 4; j <= 64; j *= 2) {
+            m_costf = (log2f((float)(j + 1)) + 1.718f) * lambda;
+            m_cost = (int)m_costf;
+            vme_state_message[MODE_INTER_MV0 + mv_count] = intel_format_lutvalue(m_cost, 0x6f);
+            mv_count++;
+        }
+
+        if (qp <= 25) {
+            vme_state_message[MODE_INTRA_16X16] = 0x4a;
+            vme_state_message[MODE_INTRA_8X8] = 0x4a;
+            vme_state_message[MODE_INTRA_4X4] = 0x4a;
+            vme_state_message[MODE_INTRA_NONPRED] = 0x4a;
+            vme_state_message[MODE_INTER_16X16] = 0x4a;
+            vme_state_message[MODE_INTER_16X8] = 0x4a;
+            vme_state_message[MODE_INTER_8X8] = 0x4a;
+            vme_state_message[MODE_INTER_8X4] = 0x4a;
+            vme_state_message[MODE_INTER_4X4] = 0x4a;
+            vme_state_message[MODE_INTER_BWD] = 0x2a;
+            return;
+        }
+        m_costf = lambda * 10;
+        vme_state_message[MODE_INTRA_16X16] = intel_format_lutvalue(m_cost, 0x8f);
+        m_cost = lambda * 14;
+        vme_state_message[MODE_INTRA_8X8] = intel_format_lutvalue(m_cost, 0x8f);
+        m_cost = lambda * 24;
+        vme_state_message[MODE_INTRA_4X4] = intel_format_lutvalue(m_cost, 0x8f);
+        m_costf = lambda * 3.5;
+        m_cost = m_costf;
+        vme_state_message[MODE_INTRA_NONPRED] = intel_format_lutvalue(m_cost, 0x6f);
+        if (slice_type == HEVC_SLICE_P) {
+            m_costf = lambda * 2.5;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_16X16] = intel_format_lutvalue(m_cost, 0x8f);
+            m_costf = lambda * 4;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_16X8] = intel_format_lutvalue(m_cost, 0x8f);
+            m_costf = lambda * 1.5;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_8X8] = intel_format_lutvalue(m_cost, 0x6f);
+            m_costf = lambda * 3;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_8X4] = intel_format_lutvalue(m_cost, 0x6f);
+            m_costf = lambda * 5;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_4X4] = intel_format_lutvalue(m_cost, 0x6f);
+            /* BWD is not used in P-frame */
+            vme_state_message[MODE_INTER_BWD] = 0;
+        } else {
+            m_costf = lambda * 2.5;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_16X16] = intel_format_lutvalue(m_cost, 0x8f);
+            m_costf = lambda * 5.5;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_16X8] = intel_format_lutvalue(m_cost, 0x8f);
+            m_costf = lambda * 3.5;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_8X8] = intel_format_lutvalue(m_cost, 0x6f);
+            m_costf = lambda * 5.0;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_8X4] = intel_format_lutvalue(m_cost, 0x6f);
+            m_costf = lambda * 6.5;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_4X4] = intel_format_lutvalue(m_cost, 0x6f);
+            m_costf = lambda * 1.5;
+            m_cost = m_costf;
+            vme_state_message[MODE_INTER_BWD] = intel_format_lutvalue(m_cost, 0x6f);
+        }
+    }
+}
